package cn.com.ctop.toutiao.modules.material.controller;

import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.toutiao.modules.material.entity.ByteDanceAdvertisePlan;
import cn.com.ctop.toutiao.modules.material.service.IByteDanceAdvertisePlanService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 今日头条广告计划信息
 */
@Slf4j
@RestController
@RequestMapping("/ctop/byteDanceAdvertisePlan")
public class ByteDanceAdvertisePlanController {

    @Autowired
    ICtopOauthTokenService ctopOauthTokenService;

    @Autowired
    private IByteDanceAdvertisePlanService byteDanceAdvertisePlanService;

    /**
     * 分页列表查询
     */
    @GetMapping(value = "/list")
    public Result<IPage<ByteDanceAdvertisePlan>> queryPageList(ByteDanceAdvertisePlan byteDanceAdvertisePlan,
                                                               @RequestParam(name = "accountId", defaultValue = "") String accountId,
                                                               @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                               @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                               HttpServletRequest req) {
        Result<IPage<ByteDanceAdvertisePlan>> result = new Result<>();
        if(!accountId.isEmpty()){
            byteDanceAdvertisePlan.setAccountId(accountId);
        }
        QueryWrapper<ByteDanceAdvertisePlan> queryWrapper = QueryGenerator.initQueryWrapper(byteDanceAdvertisePlan, req.getParameterMap());
        Page<ByteDanceAdvertisePlan> page = new Page<>(pageNo, pageSize);
        IPage<ByteDanceAdvertisePlan> pageList = byteDanceAdvertisePlanService.page(page, queryWrapper);
        //处理计划投放状态
        if(!pageList.getRecords().isEmpty()){
            pageList.getRecords().forEach(it->{
                if(it.getToutiaoStatus().equals("AD_STATUS_DELIVERY_OK")){
                    it.setStatus(true);
                }else{
                    it.setStatus(false);
                }
            });
        }
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     */
    @PostMapping(value = "/add")
    public Result<ByteDanceAdvertisePlan> add(@RequestBody ByteDanceAdvertisePlan byteDanceAdvertisePlan,
                                              @RequestParam String landingType) {
        Result<ByteDanceAdvertisePlan> result = new Result<>();
        try {
            Map<String, Object> advertiserPlan = byteDanceAdvertisePlanService.createAdvertiserPlan(ctopOauthTokenService.getTokenByAccountId(Long.valueOf(byteDanceAdvertisePlan.getAccountId())), landingType, byteDanceAdvertisePlan);
            if(advertiserPlan.get("code").equals(0)){
                byteDanceAdvertisePlanService.save(byteDanceAdvertisePlan);
                result.success("添加成功！");
            }else {
                result.error500(advertiserPlan.get("message").toString());
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     */
    @PostMapping(value = "/edit")
    public Result<ByteDanceAdvertisePlan> edit(@RequestBody ByteDanceAdvertisePlan byteDanceAdvertisePlan) {
        Result<ByteDanceAdvertisePlan> result = new Result<>();
        ByteDanceAdvertisePlan byteDanceAdvertisePlanEntity = byteDanceAdvertisePlanService.getById(byteDanceAdvertisePlan.getId());
        if (byteDanceAdvertisePlanEntity == null) {
            result.error500("未找到数据");
        } else {
            boolean ok = byteDanceAdvertisePlanService.updateById(byteDanceAdvertisePlan);
            if (ok) {
                result.success("修改成功!");
            }else {
                result.error500("未知错误");
            }
        }

        return result;
    }

    @PostMapping(value = "/batchEdit")
    public Result<List<ByteDanceAdvertisePlan>> edit(@RequestBody List<ByteDanceAdvertisePlan> byteDanceAdvertisePlanList) {
        Result<List<ByteDanceAdvertisePlan>> result = new Result<>();
        byteDanceAdvertisePlanList.forEach(it->{
            ByteDanceAdvertisePlan byteDanceAdvertisePlanEntity = byteDanceAdvertisePlanService.getById(it.getId());
            if (byteDanceAdvertisePlanEntity == null) {
                result.error500("未找到数据");
            } else {
                boolean ok = byteDanceAdvertisePlanService.updateById(it);
                if (ok) {
                    result.success("修改成功!");
                }else {
                    result.error500("未知错误");
                }
            }
        });
        result.setResult(byteDanceAdvertisePlanList);
        return result;
    }

    /**
     * 通过id删除
     */
    @PostMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            byteDanceAdvertisePlanService.removeById(id);
        } catch (Exception e) {
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     */
    @PostMapping(value = "/deleteBatch")
    public Result<ByteDanceAdvertisePlan> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ByteDanceAdvertisePlan> result = new Result<>();

        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.byteDanceAdvertisePlanService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     */
    @GetMapping(value = "/queryById")
    public Result<ByteDanceAdvertisePlan> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ByteDanceAdvertisePlan> result = new Result<>();
        ByteDanceAdvertisePlan byteDanceAdvertisePlan = byteDanceAdvertisePlanService.getById(id);
        if (byteDanceAdvertisePlan == null) {
            result.error500("未找到数据");
        } else {
            result.setResult(byteDanceAdvertisePlan);
            result.setSuccess(true);
        }
        return result;
    }
}
